﻿using DataModel.Database;
using Processor;
using UI_WinForms.Dialogs;
using UI_WinForms.Resources;

namespace UI_WinForms.Components;

public partial class TextElements : UserControl
{
    private Section? currentSection;
    private TextElement? currentTextElement;

    public TextElements()
    {
        this.InitializeComponent();
        
        // Check if we are in the designer:
        if(Program.SERVICE_PROVIDER is null)
            return;

        // Create an image list from a resource:
        var imgList = new ImageList();
        imgList.ImageSize = new Size(45, 45);
        imgList.ColorDepth = ColorDepth.Depth32Bit;
        imgList.Images.Add(Icons.icons8_align_text_left_512);
        
        // Set the image list to the list box:
        this.listTextElements.SmallImageList = imgList;

        // When the section is changed, update this component:
        AppEvents.WhenSectionChanged += async (sender, section) =>
        {
            this.currentSection = section;
            this.currentTextElement = null;
            this.buttonAdd.Enabled = this.currentSection is not null;
            this.buttonRename.Enabled = this.buttonRemove.Enabled = false;
            
            if (this.currentSection is null)
                return;
            
            // Update the path:
            this.labelSectionPath.Text = await SectionProcessor.GetSectionPath(this.currentSection.DataKey);
            await this.LoadTextElements();
        };
        
        // When the text element is changed, update the button states:
        AppEvents.WhenTextElementChanged += (sender, textElement) =>
        {
            this.currentTextElement = textElement;
            this.buttonRename.Enabled = this.buttonRemove.Enabled = this.currentTextElement is not null;
        };
    }
    
    // Loads all the text elements for the current section.
    private async Task LoadTextElements()
    {
        if (this.currentSection is null)
            return;
        
        // Load the text elements:
        var textElements = await TextElementProcessor.GetTextElements(this.currentSection, this.textBoxFilter.Text);
        
        // Update the list:
        this.listTextElements.Items.Clear();
        foreach (var textElement in textElements)
            this.listTextElements.Items.Add(new ListViewItem(textElement.Name, 0)
            {
                Tag = textElement.Id,
            });
        
        this.column.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
    }
    
    private async void buttonAdd_Click(object sender, EventArgs e)
    {
        if(this.DesignMode)
            return;
        
        var result = InputDialog.Show(new InputDialog.Options(
            Message: "Please type the desired text element's name.",
            Title: "Add a text element",
            Placeholder: "My text element",
            ShowQuestionCheckbox: false
        ));
        
        if(result.DialogResult == DialogResult.Cancel)
            return;
        
        // Add the text element to the database into the current section:
        var newTextElement = await TextElementProcessor.AddTextElement(this.currentSection?.DataKey, result.Text);
        newTextElement.ProcessError();
        
        if(!newTextElement.Successful)
            return;
        
        // Add the text element to the list:
        this.listTextElements.Items.Add(new ListViewItem(newTextElement.Result!.Name, 0)
        {
            Tag = newTextElement.Result.Id,
        });
        
        this.column.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
    }

    private async void buttonRemove_Click(object sender, EventArgs e)
    {
        if(this.DesignMode || this.currentTextElement is null)
            return;

        // Ask the user, if he really wants to remove the text element:
        if(MessageBox.Show(this.currentTextElement.Translations.Count > 0 ? $"Are you sure, you want to remove the text element '{this.currentTextElement.Name}', its {this.currentTextElement.Translations.Count} translations and so on?" : $"Are you sure, you want to remove the text element '{this.currentTextElement.Name}'?", "Remove text element", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.No)
            return;
        
        // Remove the text element1 from the database:
        await TextElementProcessor.DeleteTextElement(this.currentTextElement.Id);

        // Reload the data:
        await this.LoadTextElements();
    }
    
    private async void buttonRename_Click(object sender, EventArgs e)
    {
        if(this.DesignMode)
            return;
        
        // Ask the user if he really wants to rename the text element:
        if(MessageBox.Show("Are you sure, you want to rename the selected text element? If you are already using this element in your code, you will need to manually refactor your code after renaming it.", "Rename text element", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.No)
            return;

        // Ask the user for the new name:
        var result = InputDialog.Show(new InputDialog.Options(
            Message: "Please edit the text element's name.",
            PreloadedText: this.currentTextElement!.Name,
            ShowQuestionCheckbox: false,
            Title: "Rename text element"
        ));
        
        // If the user canceled, return:
        if(result.DialogResult == DialogResult.Cancel)
            return;
        
        // Rename the text element:
        var alteredTextElement = await TextElementProcessor.RenameTextElement(this.currentTextElement.Id, result.Text);
        
        alteredTextElement.ProcessError();
        if(!alteredTextElement.Successful)
            return;
        
        // Reload the text elements:
        await this.LoadTextElements();
    }

    private async void listTextElements_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
    {
        // Load the text element:
        var selectedTextElementId = (int)e.Item.Tag;
        this.currentTextElement = await TextElementProcessor.LoadTextElement(selectedTextElementId);
        
        // Fire the event:
        AppEvents.TextElementChanged(this.currentTextElement);
    }

    private async void textBoxFilter_KeyUp(object sender, KeyEventArgs e) => await this.LoadTextElements();
}